package de.hsbo.fbv.bmg.topology.algorithms.demos;


import java.util.ArrayList;
import java.util.List;

import de.hsbo.fbv.bmg.topology.networks.Edge;
import de.hsbo.fbv.bmg.topology.networks.Network1T;
import de.hsbo.fbv.bmg.topology.networks.Node;

/**
 * Diese Klasse dient zur schreittweisen Ausfhrung des Dijkstra-Algorithmus 
 * und enthlt daher einen Teil der Methdoen von Klasse Algorithms. 
 * Die entsprechenden Methoden wurden fr die Demonstration angepasst.
 * @author A.Zimmermann, HS Bochum, 2011
 *
 */
public class AlgorithmDemo {

	public static final int NONE = 0;
	public static final int WORK = 1;
	public static final int DONE = 2;	
	public static List<Node> queue = new ArrayList<Node>();
	public static Node nwork, nnext;
	
	/**
	 * Modifikation des Dijkstra-Algorithmus fr Demo-Zwecke: 
	 * Fhrt nur Anzahl von steps Schritten aus und beendet dann.
	 * @param network
	 * @param start
	 * @param end
	 * @param steps
	 * @return
	 */
	public static synchronized Network1T runDijkstra(Network1T network, Object start, Object end, int steps) {
		
		// Demo: Anzahl der Schritte
		int count = 1;
		
		// Wenn Knoten start nicht zum Netzwerk gehrt -> Abbruch
		if(!network.containsNode(start) || !network.containsNode(end))
			return null;
		
		// Sonst fhre Algorithmus durch
		// Bereite alle Knoten vor
		Node[] nodes = network.getNodes();
		for(int i=0; i<nodes.length; i++) {
			nodes[i].setState(NONE);
			nodes[i].setWeight(Double.MAX_VALUE);
			nodes[i].setTrace(null);
		}
		// Setze Gewicht, Spur (Trace) und Status des Startknotens
		Node nstart = network.getNode(start);
		nstart.setWeight(0);
		nstart.setState(WORK);
		nstart.setTrace(nstart);
		// Setze Endknoten
		Node nend = network.getNode(end);
		
		// Richte Bearbeitungsliste ein
		queue = new ArrayList<Node>();
		queue.add(nstart);
		
		// Durchlaufe Schleife solange, bis die Bearbeitungsliste leer ist
		do {
			// Ergnzung fr den Dijkstra-Algorithmus: Suche in Liste queue
			// nach Knoten mit minimalem Gewicht und setzte ihn an Index 0
			getMinWeight(queue);
			// Entnehme den nchsten Knoten aus der Liste und setze Status auf DONE
			nwork = queue.remove(0);
			nwork.setState(DONE);
			// Suche dazu alle Kanten und bearbeite sie
			Edge[] edges = nwork.getEdges();
			for(int i=0; i<edges.length; i++) {
				
				// Demo: Anzahl Schritte erreicht?
				if(count<steps)
					count++;
				else {
					// Ergebnis zurckgeben
					return network.getSubnet();
				}
				
				// nchsten Knoten suchen
				nnext = edges[i].getOppositeNode(nwork);
				// Knoten wurde noch nicht besucht
				if(nnext.getState()==NONE) {
					nnext.setWeight(edges[i].getWeight()+nwork.getWeight());
					nnext.setTrace(edges[i]);
					nnext.setState(WORK);
					queue.add(nnext);
				}
				// Knoten ist in der aktuellen Bearbeitungsliste
				else if(nnext.getState()==WORK) {
					double weight = edges[i].getWeight()+nwork.getWeight();
					if(weight<nnext.getWeight()) {
						nnext.setWeight(weight);
						nnext.setTrace(edges[i]);
					}
				}
				// Knoten ist bereits vollstndig bearbeitet (DONE)
				else;
			}
		} while(!queue.isEmpty() && nend.getState()!=DONE);

		// Ergebnis zurckgeben
		return network.getSubnet(end);
	}
	
	
	private static void getMinWeight(List<Node> list) {
		if(list.size()<=1)
			return;
		else {
			double weight = list.get(0).getWeight();
			for(int i=1; i<list.size(); i++) {
				if(list.get(i).getWeight()<weight) {
					list.add(0, list.remove(i));
					weight = list.get(0).getWeight();
				}
			}
		}	
	}
	
}
