package de.hsbo.fbv.bmg.topology.algorithms.demos;

import java.util.Arrays;

import de.hsbo.fbv.bmg.topology.algorithms.Algorithms;
import de.hsbo.fbv.bmg.topology.networks.Network1T;
import de.hsbo.fbv.bmg.topology.algorithms.demos.SimpleNetworkFactory;



/**
 * Kleines Demoprogramm, das fr den Graphen "Das-ist-das-Haus-vom-Ni-ko-laus" und fr ein 
 * Pentagramm, bei dem alle Knoten miteinander durch Kanten verbunden sind, den DFS-Algorithmus 
 * durchfhrt. Der Algorithmus liefert die Kanten, an denen eine Zyklus (Masche) festgestellt
 * wurde. Durch Anwenden der Eulerschen Formel lsst sich feststellen, ob der Graph planar ist. 
 * @author zimmermann
 *
 */
public class DepthFirstSearchDemo {

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		Network1T<Integer, String> hausVomKlaus = SimpleNetworkFactory.dasHausvomKlaus();
		System.out.println("Graph: Das-ist-das-Haus-vom-Ni-ko-laus");
		printEulerResult(hausVomKlaus);
		
		Network1T<Integer, Character> pentagraph = SimpleNetworkFactory.pentagraph();
		System.out.println("\nGraph: Pentagraph, alle Knoten sind miteinander verknpft");
		printEulerResult(pentagraph);
		
		Network1T<Integer, Character> zweiRechtecke = SimpleNetworkFactory.zweiRechtecke();
		System.out.println("\nGraph: Zwei getrennte Rechtecke");
		printEulerResult(zweiRechtecke);
		
		Network1T<Integer, Character> verbRechtecke = SimpleNetworkFactory.verdundeneRechtecke();
		System.out.println("\nGraph: Zwei verbundene Rechtecke");
		printEulerResult(verbRechtecke);
	}
	
	public static void printEulerResult(Network1T network) {
		// Depth-First-Search ausfhren
		Network1T cycles = Algorithms.runDepthFirstSearch(network);
		// Anzahl Knoten, Kanten und Zyklen ermitteln 
		int nNodes = network.getNodes().length;
		int nEdges = network.getEdges().length;
		int nCycles = cycles.getEdges().length;
		// Anwendung der Eulerschen Formel: 
		int nEuler = nNodes - nEdges + nCycles + 1;
		// Ergebnisse ausgeben
		System.out.println("Anzahl der Knoten v: "+ nNodes);
		System.out.println("Anzahl der Kanten e: "+ nEdges);
		System.out.println("Anzahl der Zyklen f: "+ nCycles +" (ermittelt mit dem DFS-Algorithmus)");
		System.out.println("Anwendung der Eulerschen Formel: c = (v - e + f+1) : " + nEuler) ;
		if(nEuler<=2)
			System.out.println("Ergebnis: Der Graph ist zusammenhngend.");
		else
			System.out.println("Ergebnis: Der Graph ist NICHT zusammenhngend.");
	}

}
