package de.hsbo.fbv.bmg.topology.algorithms.demos;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.JPanel;
import javax.swing.JFrame;
import java.awt.GridBagLayout;
import java.awt.Dimension;
import javax.swing.JLabel;
import java.awt.GridBagConstraints;
import javax.swing.JComboBox;
import javax.swing.JButton;

import de.hsbo.fbv.bmg.graphics.viewer.simple.Map;
import de.hsbo.fbv.bmg.topology.algorithms.Algorithms;
import de.hsbo.fbv.bmg.topology.networks.Network1T;
import de.hsbo.fbv.bmg.topology.algorithms.demos.Place;
import de.hsbo.fbv.bmg.topology.algorithms.demos.Way;
import de.hsbo.fbv.bmg.topology.networks.impl.SimpleElement;
import de.hsbo.fbv.bmg.topology.networks.impl.SimpleNetworkLayer;


import java.awt.Insets;
import java.awt.FlowLayout;
import java.util.Arrays;
import java.util.Set;

/**
 * Demo-Programm zum Dijkstra-Algorithmus
 * @author A.Zimmermann, HS Bochum, 2011
 *
 */
public class DijkstraDemo extends JFrame {
	
	public static void main(String[] args) {
		DijkstraDemo nwf = new DijkstraDemo();
		nwf.setVisible(true);
	}

	
	// Attribute
	Map map = new Map();
	JFrame thisframe = null;
	SimpleNetworkLayer<Place, Way> orig;  //  @jve:decl-index=0:
	SimpleNetworkLayer treeL;
	SimpleNetworkLayer nodeL;
	int step = 0;

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JPanel jTextPanel = null;
	private JLabel jLabelStart = null;
	private JComboBox jComboBoxStart = null;
	private JLabel jLabelEnd = null;
	private JComboBox jComboBoxEnd = null;
	private JButton jButton = null;
	private JPanel jPanel = null;  //  @jve:decl-index=0:visual-constraint="590,10"
	private JLabel jLabel = null;
	private JButton jButton1 = null;
	private JLabel jLabelactNodes = null;

	/**
	 * This is the default constructor
	 */
	public DijkstraDemo() {
		super();
		initialize();
		thisframe = this;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(570, 400);
		this.setContentPane(getJContentPane());
		this.setTitle("Dijkstra");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(map, BorderLayout.CENTER);
			jContentPane.add(getJTextPanel(), BorderLayout.NORTH);
			jContentPane.add(getJPanel(), BorderLayout.SOUTH);
			orig = NetworkFactory.getNrwCities();
			orig.setNodeColor(Color.WHITE);
			map.add(orig);
			Set<Place> set = orig.getNodeObjects();
			Object[] places = set.toArray();
			Arrays.sort(places);
			for(int i=0; i<places.length; i++)
				jComboBoxStart.addItem(places[i]);
			for(int i=0; i<places.length; i++)
				jComboBoxEnd.addItem(places[i]);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jTextPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJTextPanel() {
		if (jTextPanel == null) {
			jLabelEnd = new JLabel();
			jLabelEnd.setText("Ende");
			jLabelStart = new JLabel();
			jLabelStart.setText("Start");
			jTextPanel = new JPanel();
			jTextPanel.setLayout(new FlowLayout());
			jTextPanel.add(jLabelStart, null);
			jTextPanel.add(getJComboBoxStart(), null);
			jTextPanel.add(jLabelEnd, null);
			jTextPanel.add(getJComboBoxEnd(), null);
			jTextPanel.add(getJButton(), null);
			jTextPanel.add(getJButton1(), null);
		}
		return jTextPanel;
	}

	/**
	 * This method initializes jComboBoxStart	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getJComboBoxStart() {
		if (jComboBoxStart == null) {
			jComboBoxStart = new JComboBox();
			jComboBoxStart.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					map.repaint();
				}
			});
		}
		return jComboBoxStart;
	}

	/**
	 * This method initializes jComboBoxEnd	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getJComboBoxEnd() {
		if (jComboBoxEnd == null) {
			jComboBoxEnd = new JComboBox();
			jComboBoxEnd.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					map.repaint();
				}
			});
		}
		return jComboBoxEnd;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setText("OK");
			jButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					Place start = (Place)jComboBoxStart.getSelectedItem();
					Place end = (Place)jComboBoxEnd.getSelectedItem();
					Network1T<Place, Way> tree = Algorithms.runDijkstra(orig, start, end);
					// Network<Place, Way> tree = Algorithms.runDijkstra(orig, start);
					if(map.contains(treeL))
						map.remove(treeL);
					treeL = new SimpleNetworkLayer(tree);
					treeL.setEdgeStroke(new BasicStroke(2.f));
					treeL.setEdgeColor(Color.RED);
					treeL.setNodeColor(Color.RED);
					treeL.setNodeStroke(new BasicStroke(4.f));
					map.add(treeL);
					thisframe.repaint();
				}
			});
		}
		return jButton;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints1.gridy = 1;
			gridBagConstraints1.anchor = GridBagConstraints.WEST;
			gridBagConstraints1.gridx = 0;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.insets = new Insets(5, 5, 5, 5);
			gridBagConstraints.gridy = 0;
			gridBagConstraints.anchor = GridBagConstraints.WEST;
			gridBagConstraints.gridx = 0;
			jLabelactNodes = new JLabel();
			jLabelactNodes.setText("Bearbeitung: Startknoten - Endknoten");
			jLabel = new JLabel();
			jLabel.setText("Bearbeitungsschlange: ");
			jPanel = new JPanel();
			jPanel.setLayout(new GridBagLayout());
			jPanel.setSize(new Dimension(194, 34));
			jPanel.add(jLabelactNodes, gridBagConstraints);
			jPanel.add(jLabel, gridBagConstraints1);
		}
		return jPanel;
	}

	/**
	 * This method initializes jButton1	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton1() {
		if (jButton1 == null) {
			jButton1 = new JButton();
			jButton1.setText("Schritte");
			jButton1.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					Place start = (Place)jComboBoxStart.getSelectedItem();
					Place end = (Place)jComboBoxEnd.getSelectedItem();
					step++;
					Network1T<Place, Way> tree = AlgorithmDemo.runDijkstra(orig, start, end, step);
					SimpleElement.setPrintoptions(SimpleElement.OBJECT + SimpleElement.WEIGHT + SimpleElement.STATE);
					jLabelactNodes.setText("Bearbeitung: Startknoten "+AlgorithmDemo.nwork + " - Endknoten "+AlgorithmDemo.nnext );
					SimpleElement.setPrintoptions(SimpleElement.OBJECT + SimpleElement.WEIGHT);
					jLabel.setText("Bearbeitungsschlange: "+AlgorithmDemo.queue);
					if(map.contains(treeL))
						map.remove(treeL);
					treeL = new SimpleNetworkLayer(tree);
					treeL.setEdgeColor(Color.BLUE);
					treeL.setNodeColor(Color.BLUE);
					map.add(treeL);
					if(map.contains(nodeL))
						map.remove(nodeL);
					nodeL = new SimpleNetworkLayer();
					if(AlgorithmDemo.nwork.getObject()!=null)
						nodeL.addNode(AlgorithmDemo.nwork.getObject());
					/*
					if(AlgorithmDemo.nnext.getObject()!=null)
						nodeL.addNode(AlgorithmDemo.nnext.getObject()); */
					nodeL.setNodeColor(Color.RED);
					map.add(nodeL);
					thisframe.repaint();
				}
			});
		}
		return jButton1;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
