package de.hsbo.fbv.bmg.topology.algorithms.demos;

import de.hsbo.fbv.bmg.geometry.simple.CoordinateM;
import de.hsbo.fbv.bmg.topology.networks.Network1T;
import de.hsbo.fbv.bmg.topology.algorithms.demos.Place;
import de.hsbo.fbv.bmg.topology.algorithms.demos.Way;
import de.hsbo.fbv.bmg.topology.networks.impl.SimpleNetwork;
import de.hsbo.fbv.bmg.topology.networks.impl.SimpleNetworkLayer;


public class NetworkFactory {
	
	/**
	 * Generiert das Knoten-Kanten-Netz "Das ist das Haus vom Nikolaus".
	 * @return
	 */
	public static SimpleNetworkLayer<Place, Way> getHausVomKlaus() {
		
		Network1T<Place, Way> net = new SimpleNetwork<Place, Way>();
		
		Way[] ways;
		try {
			Place p1 = new Place("1", new CoordinateM(100.0, 100.0));
			Place p2 = new Place("2", new CoordinateM(100.0, 200.0));
			Place p3 = new Place("3", new CoordinateM(150.0, 250.0));
			Place p4 = new Place("4", new CoordinateM(200.0 ,200.0));
			Place p5 = new Place("5", new CoordinateM(200.0 ,100.0));
			ways = new Way[] {
					new Way(p5, p1),	// Das
					new Way(p1, p2),	// ist
					new Way(p2, p3), 	// das	
					new Way(p3, p4),	// Haus
					new Way(p4, p2),	// vom	
					new Way(p2, p5),	// Ni-
					new Way(p5, p4),	// ko-
					new Way(p4, p1)};	// laus
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		
		for(int i=0; i<ways.length; i++)
			net.addEdge(ways[i], ways[i].getStart(), ways[i].getEnd());
		
		SimpleNetworkLayer<Place, Way> nl = new SimpleNetworkLayer<Place, Way>(net);
		
		return nl;
	}

	/**
	 * Generiert eine Knoten-Kanten-Netz mit vereinfachten Verbindungen 
	 * zwischen verschiedenen Stdten in NRW.
	 * @return
	 */
	public static SimpleNetworkLayer<Place, Way> getNrwCities() {
		
		// Erzeugt das Netzwerk-Objekt mit den Hilfsklassen Place und Way
		Network1T<Place, Way> net = new SimpleNetwork<Place, Way>();
		
		try {
			// Legt die Stdte an
			Place[] places = new Place[] {
					// 0
					new Place("Aachen", new CoordinateM(295, 5629)),
					new Place("Kln", new CoordinateM(356, 5644)),
					new Place("Bonn", new CoordinateM(365, 5623)),
					new Place("Mnchengladbach", new CoordinateM(317, 5681)),
					new Place("Dsseldorf", new CoordinateM(344, 5676)),
					// 5
					new Place("Krefeld", new CoordinateM(333, 5689)),
					new Place("Duisburg", new CoordinateM(345, 5700)),
					new Place("Oberhausen", new CoordinateM(351, 5703)),
					new Place("Mlheim", new CoordinateM(354, 5699)),
					new Place("Bottrop", new CoordinateM(356, 5709)),
					// 10
					new Place("Gelsenkirchen", new CoordinateM(368, 5710)),
					new Place("Recklinghausen", new CoordinateM(375, 5720)),
					new Place("Herne", new CoordinateM(376, 5711)),
					new Place("Bochum", new CoordinateM(376, 5704)),
					new Place("Essen", new CoordinateM(362, 5702)),
					// 15
					new Place("Wuppertal", new CoordinateM(372, 5681)),
					new Place("Solingen", new CoordinateM(366, 5670)),
					new Place("Bergisch-Gladbach", new CoordinateM(369, 5651)),
					new Place("Remscheid", new CoordinateM(374, 5671)),
					new Place("Hagen", new CoordinateM(393, 5691)),
					// 20
					new Place("Siegen", new CoordinateM(432, 5636)),
					new Place("Mnster", new CoordinateM(405, 5758)),
					new Place("Bielefeld", new CoordinateM(468, 5764)),
					new Place("Paderborn", new CoordinateM(483, 5730)),
					new Place("Dortmund", new CoordinateM(393, 5708)),
					// 25
					new Place("Hamm", new CoordinateM(418, 5726))
				};
			
			// Legt die Stdteverbindungen an, fehlende Stdteknoten werden dabei automatisch ergnzt
			Way[] ways = new Way[] {
					new Way(places[0],places[1]), new Way(places[0],places[3]), // Aachen
					new Way(places[1],places[4]), new Way(places[1],places[2]), new Way(places[1],places[17]), new Way(places[1],places[18]), // Kln
					new Way(places[2],places[20]), // Bonn
					new Way(places[3],places[4]), new Way(places[3],places[5]), // Mnchengladbach
					new Way(places[4],places[5]), new Way(places[4],places[6]), new Way(places[4],places[16]), // Dsseldorf
					new Way(places[5],places[6]), // Krefeld
					new Way(places[6],places[7]), new Way(places[6],places[8]), new Way(places[6],places[9]), // Duisburg
					new Way(places[7],places[9]), new Way(places[7],places[14]), new Way(places[7],places[8]), // Oberhausen
					new Way(places[8],places[14]), // Mlheim
					new Way(places[9],places[10]), new Way(places[9],places[14]), // Bottrop
					new Way(places[10],places[11]), new Way(places[10],places[12]), new Way(places[10],places[13]), new Way(places[10],places[14]), // Gelsenkirchen
					new Way(places[11],places[21]), new Way(places[11],places[12]), new Way(places[11],places[24]), // Recklinghausen
					new Way(places[12],places[13]), new Way(places[12],places[24]), // Herne
					new Way(places[13],places[14]), new Way(places[13],places[15]), new Way(places[13],places[19]), new Way(places[13],places[24]), // Bochum
					new Way(places[14],places[15]), // Essen
					new Way(places[15],places[16]), new Way(places[15],places[18]), new Way(places[15],places[19]), // Wuppertal
					new Way(places[16],places[18]), // Solingen
					// Bergisch-Gladbach
					// Remscheid
					new Way(places[19],places[24]), new Way(places[19],places[20]), // Hagen
					// Siegen
					new Way(places[21],places[22]), new Way(places[21],places[25]), // Mnster
					new Way(places[22],places[23]), new Way(places[21],places[25]), // Bielefeld
					new Way(places[23],places[25]), // Paderborn
					new Way(places[24],places[25]) // Dortmund
					// Hamm
					};
			
			// Fgt alle Stdteverbindungen ins Netzwerk ein
			for(int i=0; i<ways.length; i++)
				net.addEdge(ways[i], ways[i].getStart(), ways[i].getEnd() , Math.floor(ways[i].length()));
			
			SimpleNetworkLayer<Place, Way> nl = new SimpleNetworkLayer<Place, Way>(net);
			
			return nl;
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}	// Ende Methode
	
	
	
}	// Ende Klasse
