package de.hsbo.fbv.bmg.topology.algorithms.demos;

import de.hsbo.fbv.bmg.topology.networks.Network1T;
import de.hsbo.fbv.bmg.topology.networks.impl.SimpleNetwork;

/**
 * Klasse zum Generieren verschiedener Beispiel-Graphen 
 * @author zimmermann
 *
 */
public class SimpleNetworkFactory {
	
	/**
	 * Stdte in Nordrhein-Westfalen
	 * @return
	 */
	public static Network1T<String, String> citiesNRW() {
		// Graph erzeugen
		Network1T<String, String> nrw = new SimpleNetwork<String, String>();
		// Kanten anlegen
		nrw.addEdge("AC-DU", "AC", "DU",  90);
		nrw.addEdge("AC-K",  "AC", "K",   70);
		nrw.addEdge("K-D",   "K",  "D",   40);
		nrw.addEdge("K-BN",  "K",  "BN",  30);
		nrw.addEdge("K-DO",  "K",  "DO",  80);
		nrw.addEdge("K-SI",  "K",  "SI",  80);
		nrw.addEdge("BN-SI", "BN", "SI",  90);
		nrw.addEdge("D-DU",  "D",  "DU",  30);
		nrw.addEdge("D-DO",  "D",  "DO",  60);
		nrw.addEdge("DU-E",  "DU", "E",   20);
		nrw.addEdge("E-DO",  "E",  "DO",  30);
		nrw.addEdge("E-MS",  "E",  "MS",  70);
		nrw.addEdge("DO-MS", "DO", "MS",  50);
		nrw.addEdge("DO-BI", "DO", "BI", 100);
		nrw.addEdge("DO-PB", "DO", "PB", 100);
		nrw.addEdge("DO-SI", "DO", "SI",  80);
		nrw.addEdge("MS-BI", "MS", "BI",  70);
		nrw.addEdge("BI-PB", "BI", "PB",  40);
		nrw.addEdge("SI-PB", "SI", "PB", 120);
		// Graph zurckgeben
		return nrw;
	}
	
	/**
	 * Graph "Das ist das Haus vom Nikolaus" 
	 * 8 Kanten - 5 Knoten
	 * @return
	 */
	public static Network1T<Integer, String> dasHausvomKlaus() {
		Network1T<Integer, String> dHvN = new SimpleNetwork<Integer, String>();
		dHvN.addEdge("Das",  1, 2);
		dHvN.addEdge("ist",  2, 3);
		dHvN.addEdge("das",  3, 1);
		dHvN.addEdge("Haus", 1, 4);
		dHvN.addEdge("vom",  4, 5);
		dHvN.addEdge("Ni-",  5, 3);
		dHvN.addEdge("ko-",  3, 4);
		dHvN.addEdge("laus", 4, 2);
		// Graph zurckgeben
		return dHvN;	
	}
	
	/**
	 * Graph Fnfeck, bei dem jeder Eckpunkt zu jedem anderen eine Verbindung besitzt.
	 * @return
	 */
	public static Network1T<Integer, Character> pentagraph() {
		Network1T<Integer, Character> penta = new SimpleNetwork<Integer, Character>();
		penta.addEdge('a', 1, 2);
		penta.addEdge('b', 1, 3);
		penta.addEdge('c', 1, 4);
		penta.addEdge('d', 1, 5);
		penta.addEdge('e', 2, 3);
		penta.addEdge('f', 2, 4);
		penta.addEdge('g', 2, 5);
		penta.addEdge('h', 3, 4);
		penta.addEdge('i', 3, 5);
		penta.addEdge('k', 4, 5);
		// Graph zurckgeben
		return penta;	
		
	}
	
	/**
	 * Graph mit zwei getrennten Rechtecken
	 * @return
	 */
	public static Network1T<Integer, Character> zweiRechtecke() {
		Network1T<Integer, Character> rect = new SimpleNetwork<Integer, Character>();
		rect.addEdge('a', 1, 2);
		rect.addEdge('b', 2, 3);
		rect.addEdge('c', 3, 4);
		rect.addEdge('d', 4, 1);
		rect.addEdge('e', 5, 6);
		rect.addEdge('f', 6, 7);
		rect.addEdge('g', 7, 8);
		rect.addEdge('h', 8, 5);
		// Graph zurckgeben
		return rect;	
	}
	
	/**
	 * Graph mit zwei verbundenen Rechtecken
	 * @return
	 */
	public static Network1T<Integer, Character> verdundeneRechtecke() {
		Network1T<Integer, Character> rect = new SimpleNetwork<Integer, Character>();
		rect.addEdge('a', 1, 2);
		rect.addEdge('b', 2, 3);
		rect.addEdge('c', 3, 4);
		rect.addEdge('d', 4, 1);
		rect.addEdge('e', 5, 6);
		rect.addEdge('f', 6, 7);
		rect.addEdge('g', 7, 8);
		rect.addEdge('h', 8, 5);
		rect.addEdge('i', 3, 8);
		// Graph zurckgeben
		return rect;	
	}
	
	
	

}
